@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2019 IBM Corp. and others
+ * Copyright (c) 1998, 2020 IBM Corp. and others
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -42,6 +42,7 @@ import java.lang.ref.ReferenceQueue;
 import java.lang.ref.WeakReference;
 import java.lang.reflect.*;
 import java.security.cert.Certificate;
+import sun.security.util.SecurityConstants;
 
 import java.lang.reflect.Modifier;
 import java.util.Spliterator;
@@ -53,7 +54,7 @@ import java.util.concurrent.ConcurrentHashMap;
 import jdk.internal.reflect.CallerSensitive;
 import jdk.internal.loader.ClassLoaders;
 import jdk.internal.loader.BootLoader;
- 
+
 /**
  * ClassLoaders are used to dynamically load, link and install
  * classes into a running image.
@@ -118,7 +119,7 @@ public abstract class ClassLoader {
 	private static boolean lazyClassLoaderInit = true;
 	private static boolean specialLoaderInited = false;
 	private static InternalAnonymousClassLoader internalAnonClassLoader;
-	private static native void initAnonClassLoader(InternalAnonymousClassLoader anonClassLoader);	
+	private static native void initAnonClassLoader(InternalAnonymousClassLoader anonClassLoader);
 	
 	static final class ClassNameLockRef extends WeakReference<Object> implements Runnable {
 		private static final ReferenceQueue<Object> queue = new ReferenceQueue<>();
@@ -209,7 +210,12 @@ public abstract class ClassLoader {
 		try {
 			System.bootLayer = jdk.internal.module.ModuleBootstrap.boot();
 		} catch (Exception ex) {
-			ex.printStackTrace();
+			System.out.println(ex);
+			Throwable t = ex.getCause();
+			while (t != null) {
+				System.out.println("Caused by: " + t); //$NON-NLS-1$
+				t = t.getCause();
+			}
 			System.exit(1);
 		}
 		jdk.internal.misc.VM.initLevel(2);
@@ -704,7 +710,7 @@ public final ClassLoader getParent() {
 	if (security != null) {	
 		ClassLoader callersClassLoader = callerClassLoader();
 		if (needsClassLoaderPermissionCheck(callersClassLoader, parent)) {
-			security.checkPermission(com.ibm.oti.util.RuntimePermissions.permissionGetClassLoader);
+			security.checkPermission(SecurityConstants.GET_CLASSLOADER_PERMISSION);
 		}
 	}
 	return parent;
@@ -913,7 +919,7 @@ public static ClassLoader getPlatformClassLoader() {
 	if (security != null) {
 		ClassLoader callersClassLoader = callerClassLoader();
 		if (needsClassLoaderPermissionCheck(callersClassLoader, platformClassLoader)) {
-			security.checkPermission(com.ibm.oti.util.RuntimePermissions.permissionGetClassLoader);
+			security.checkPermission(SecurityConstants.GET_CLASSLOADER_PERMISSION);
 		}
 	}
 	return platformClassLoader;
@@ -989,7 +995,7 @@ public static ClassLoader getSystemClassLoader () {
 	if (security != null) {	
 		ClassLoader callersClassLoader = callerClassLoader();
 		if (needsClassLoaderPermissionCheck(callersClassLoader, sysLoader)) {
-			security.checkPermission(com.ibm.oti.util.RuntimePermissions.permissionGetClassLoader);
+			security.checkPermission(SecurityConstants.GET_CLASSLOADER_PERMISSION);
 		}
 	}
 
@@ -1667,10 +1673,7 @@ static ClassLoader callerClassLoader() {
  * @exception	SecurityException
  *							if the library was not allowed to be loaded
  */
-static synchronized void loadLibraryWithClassLoader(String libName, ClassLoader loader) {
-	SecurityManager smngr = System.getSecurityManager();
-	if (smngr != null)
-		smngr.checkLink(libName);
+static void loadLibraryWithClassLoader(String libName, ClassLoader loader) {
 	if (loader != null) {
 		String realLibName = loader.findLibrary(libName);
 		
@@ -1738,11 +1741,13 @@ static void loadLibraryWithPath(String libName, ClassLoader loader, String libra
 		} catch (java.io.IOException e) {
 			error = com.ibm.oti.util.Util.toString(message);
 		}
-		throw new UnsatisfiedLinkError(libName + " (" + error + ")"); //$NON-NLS-1$ //$NON-NLS-2$
+		// K0649 = {0} ({1})
+		throw new UnsatisfiedLinkError(com.ibm.oti.util.Msg.getString("K0649", libName, error));//$NON-NLS-1$
+
 	}
 }
 
-private static native byte[] loadLibraryWithPath(byte[] libName, ClassLoader loader, byte[] libraryPath);
+private static synchronized native byte[] loadLibraryWithPath(byte[] libName, ClassLoader loader, byte[] libraryPath);
 
 static void loadLibrary(Class<?> caller, String name, boolean fullPath) {
 	if (fullPath)
@@ -1760,6 +1765,10 @@ static void loadLibrary(Class<?> caller, String name, boolean fullPath) {
  * @since 1.4
  */
 public void setClassAssertionStatus(String cname, boolean enable) {
+	setClassAssertionStatusImpl(cname, enable);
+}
+
+private void setClassAssertionStatusImpl(String cname, boolean enable) {
 	if (!isParallelCapable) {
 		synchronized(this) {
 			setClassAssertionStatusHelper(cname, enable);
@@ -1787,6 +1796,10 @@ private void setClassAssertionStatusHelper(final String cname, final boolean ena
  * @since 1.4
  */
 public void setPackageAssertionStatus(String pname, boolean enable) {
+	setPackageAssertionStatusImpl(pname, enable);
+}
+
+private void setPackageAssertionStatusImpl(String pname, boolean enable) {
 	if (!isParallelCapable) {
 		synchronized(this) {
 			setPackageAssertionStatusHelper(pname, enable);
@@ -1814,6 +1827,10 @@ private void setPackageAssertionStatusHelper(final String pname, final boolean e
  * @since 1.4
  */
 public void setDefaultAssertionStatus(boolean enable){
+	setDefaultAssertionStatusImpl(enable);
+}
+
+private void setDefaultAssertionStatusImpl(boolean enable){
 	if (!isParallelCapable) {
 		synchronized(this) {
 			defaultAssertionStatus = enable;
@@ -1994,16 +2011,16 @@ private void initializeClassLoaderAssertStatus() {
 					if (bootLoader) {
 						continue;
 					}
-					setDefaultAssertionStatus(def);
+					setDefaultAssertionStatusImpl(def);
 				} else {
 					String str = vmargExtraInfo;
 					int len = str.length();
 					if ( len > 3 && str.charAt(len-1) == '.'  && 
 						str.charAt(len-2) == '.' && str.charAt(len-3) == '.') {
 						str = str.substring(0,len-3);
-						setPackageAssertionStatus(str, def);
+						setPackageAssertionStatusImpl(str, def);
 					} else {
-						setClassAssertionStatus(str, def);
+						setClassAssertionStatusImpl(str, def);
 					}
 				}
 		} else if ( vmargOptions.compareTo("-esa") == 0  //$NON-NLS-1$
@@ -2013,7 +2030,7 @@ private void initializeClassLoaderAssertStatus() {
 		) {
 			if (bootLoader) {
 				boolean def = vmargOptions.charAt(1) == 'e';
-				setDefaultAssertionStatus(def);
+				setDefaultAssertionStatusImpl(def);
 			}
 		}
 
