@@ -829,6 +829,15 @@ public final class MethodType implements Serializable
 		return arguments.length;
 	}
 
+	/**
+	 * Method to return array of arguments
+	 *
+	 * @return array of arguments
+	 */
+	Class<?>[] ptypes() {
+		return arguments;
+	}
+
 	/**
 	 * Helper method to return the parameter types in a List.
 	 * <br>  
@@ -1055,6 +1064,7 @@ public final class MethodType implements Serializable
 		return wrapperClass;
 	}
 	
+	/* The string returned by this method should be in sync with Class.descriptorString() */
 	static String getBytecodeStringName(Class<?> c){
 		if (c.isPrimitive()) {
 			if (c == int.class) {
@@ -1077,7 +1087,15 @@ public final class MethodType implements Serializable
 				return "S"; //$NON-NLS-1$
 			}
 		}
+		Class<?> clazz = c;
+		if (c.isArray()) {
+			clazz = c.getComponentType();
+			while (clazz.isArray()) {
+				clazz = clazz.getComponentType();
+			}
+		}
 		String name = c.getName().replace('.', '/');
+		
 		if (c.isArray()) {
 			return name;
 		}
