@@ -245,11 +245,19 @@ public abstract class VarHandle extends VarHandleInternal
 	}
 	
 	enum AccessType {
-		GET,
-		SET,
-		COMPARE_AND_SET,
-		COMPARE_AND_EXCHANGE,
-		GET_AND_UPDATE;
+		GET(Object.class),
+		SET(void.class),
+		COMPARE_AND_SET(boolean.class),
+		COMPARE_AND_EXCHANGE(Object.class),
+		GET_AND_UPDATE(Object.class);
+
+		final boolean isMonomorphicInReturnType;
+		final Class<?> returnType;
+
+		AccessType(Class<?> returnType) {
+			this.returnType = returnType;
+			this.isMonomorphicInReturnType = (returnType != Object.class);
+		}
 
 		/**
 		 * Gets the MethodType associated with the AccessType.
@@ -305,10 +313,11 @@ public abstract class VarHandle extends VarHandleInternal
 	}
 	
 	static final Unsafe _unsafe = Unsafe.getUnsafe();
-	static final Lookup _lookup = Lookup.internalPrivilegedLookup;
+	static final Lookup _lookup = Lookup.IMPL_LOOKUP;
 
 	
 	private final MethodHandle[] handleTable;
+
 	final Class<?> fieldType;
 	final Class<?>[] coordinateTypes;
 	final int modifiers;
@@ -379,8 +388,8 @@ public abstract class VarHandle extends VarHandleInternal
 		if (internalHandle == null) {
 			modifiedType = accessModeTypeUncached(accessMode);
 		} else {
-			MethodType internalType = internalHandle.type;
-			int numOfArguments = internalType.arguments.length;
+			MethodType internalType = internalHandle.type();
+			int numOfArguments = internalType.parameterCount();
 
 			/* Drop the internal VarHandle argument. */
 			modifiedType = internalType.dropParameterTypes(numOfArguments - 1, numOfArguments);
@@ -462,14 +471,14 @@ public abstract class VarHandle extends VarHandleInternal
 		MethodHandle mh = handleTable[accessMode.ordinal()];
 
 		if (mh != null) {
-			mh = MethodHandles.insertArguments(mh, mh.type.parameterCount() - 1, this);
+			mh = MethodHandles.insertArguments(mh, mh.type().parameterCount() - 1, this);
 		}
 
 		if ((mh == null) || !isAccessModeSupported(accessMode)) {
 			MethodType mt = null;
 
 			if (mh != null) {
-				mt = mh.type;
+				mt = mh.type();
 			} else {
 				mt = accessModeTypeUncached(accessMode);
 				/* accessModeTypeUncached does not return null. It throws InternalError if the method type
@@ -931,13 +940,15 @@ public abstract class VarHandle extends VarHandleInternal
 		try {
 			/* Lookup the MethodHandles corresponding to access modes. */
 			for (AccessMode accessMode : AccessMode.values()) {
-				operationMHs[accessMode.ordinal()] = _lookup.findStatic(operationsClass, accessMode.methodName(), lookupTypes[accessMode.ordinal()]);
+				int index = accessMode.ordinal();
+				operationMHs[index] = _lookup.findStatic(operationsClass, accessMode.methodName(), lookupTypes[index]);
 			}
 			
 			/* If we provided a different set of exactTypes, clone the MethodHandles with the exact types. */
 			if (lookupTypes != exactTypes) {
 				for (AccessMode accessMode : AccessMode.values()) {
-					operationMHs[accessMode.ordinal()] = operationMHs[accessMode.ordinal()].cloneWithNewType(exactTypes[accessMode.ordinal()]);
+					int index = accessMode.ordinal();
+					operationMHs[index] = operationMHs[index].cloneWithNewType(exactTypes[index]);
 				}
 			}
 		} catch (IllegalAccessException | NoSuchMethodException e) {
