@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2017, 2019 IBM Corp. and others
+ * Copyright (c) 2017, 2020 IBM Corp. and others
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -20,6 +20,12 @@
  * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
  *******************************************************************************/
 
+/**
+ * Provides access to CUDA-capable devices from Java.
+ * <p>
+ * A wealth of background information on CUDA is available <a href="http://www.nvidia.com/object/cuda_home.html">here</a>.
+ */
+@SuppressWarnings("module")
 module openj9.cuda {
   requires java.base;
   exports com.ibm.cuda;
